"""Unit Tests for the socialprofile module forms"""

# pylint: disable=R0904, C0103

from django.test import TestCase

from django.contrib.auth.models import User
from social.apps.django_app.default.models import UserSocialAuth
from socialprofile.forms import SocialProfileForm, UserForm
from django.forms.models import model_to_dict
import logging

LOGGER = logging.getLogger(name='socialprofile.test_forms')


class SocialProfileFormTestCase(TestCase):
    """Test Case for Social Profile Forms"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfileForm Tests setUp")
        self.user1 = User.objects.create_user('user1', 'user1@user1.com', 'user1password')
        self.user1.social_profile.gender = 'other'
        self.user1.social_profile.url = 'http://test.com'
        self.user1.social_profile.description = 'Test User 1'
        self.user1.social_profile.image_url = 'http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm'
        self.user1.save()
        self.sa1 = UserSocialAuth.objects.create(user=self.user1, provider='google-oauth2', uid='user1@user1.com')

    def test_socialprofile_form_view(self):
        """Test editing user profile data through form in isolation"""
        LOGGER.debug("Test socialprofile edit form")
        form = SocialProfileForm(instance=self.user1.social_profile)
        form_html = form.as_p()
        LOGGER.debug(form_html)
        self.assertInHTML('<option value="other" selected="selected">Other</option>', form_html)
        self.assertInHTML('<input id="id_returnTo" name="returnTo" type="hidden" value="/" />', form_html)
        self.assertInHTML('<textarea cols="40" id="id_description" name="description" rows="10">Test User 1</textarea>', form_html)
        self.assertInHTML('<input id="id_image_url" maxlength="500" name="image_url" type="url" value="http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm" />', form_html)
        self.assertInHTML('<input id="id_manually_edited" name="manually_edited" type="hidden" value="True" />', form_html)
        self.assertInHTML('<input id="id_url" maxlength="500" name="url" type="url" value="http://test.com" />', form_html)

    def test_socialprofile_form_update(self):
        LOGGER.debug("Test socialprofile edit form")
        data = model_to_dict(self.user1.social_profile)
        data['description'] = 'new description'
        data['gender'] = 'female'
        data['url'] = 'http://new.url'
        data['image_url'] = 'http://new.image.url'
        form = SocialProfileForm(data=data, instance=self.user1.social_profile)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEquals(self.user1.social_profile.description, 'new description')
        self.assertEquals(self.user1.social_profile.url, 'http://new.url')
        self.assertEquals(self.user1.social_profile.gender, 'female')
        self.assertEquals(self.user1.social_profile.image_url, 'http://new.image.url')

    def test_socialprofile_form_clean_desc(self):
        LOGGER.debug("Test socialprofile form clean desc")
        data = model_to_dict(self.user1.social_profile)
        data['description'] = '<a href="http://bad.url">Bad Link</a>'
        form = SocialProfileForm(data=data, instance=self.user1.social_profile)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEquals(self.user1.social_profile.description, 'Bad Link')


class UserFormTestCase(TestCase):
    """Tests for  User Form"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("UserForm Tests setUp")
        self.user1 = User.objects.create_user('user1', 'user1@user1.com', 'user1password')

    def test_user_form_create(self):
        LOGGER.debug("Test user form create")
        data = {
            'username': 'user2',
            'password': 'user2password',
            'email': 'user@user2.com',
            'first_name': 'user',
            'last_name': '2',
            'last_login': '2012-09-04 06:00',
            'date_joined': '2012-09-04 06:00',
            'manually_edited': False
        }
        form = UserForm(data)
        self.assertTrue(form.is_valid())
        form.save()
        user2 = User.objects.get(username='user2')
        self.assertEquals(user2.username, 'user2')

    def test_user_form_view(self):
        """Test editing user profile data through form in isolation"""
        LOGGER.debug("Test user edit form")
        form = UserForm(instance=self.user1)
        form_html = form.as_p()
        LOGGER.debug(form_html)
        self.assertInHTML('<input id="id_username" maxlength="30" name="username" type="text" value="user1" />', form_html)
        self.assertInHTML('<input id="id_first_name" maxlength="30" name="first_name" type="text" />', form_html)
        self.assertInHTML('<input id="id_email" maxlength="254" name="email" type="email" value="user1@user1.com" />', form_html)
        self.assertInHTML('<input id="id_last_name" maxlength="30" name="last_name" type="text" />', form_html)
        self.assertNotIn('<input id="id_is_staff" name="is_staff" type="checkbox" />', form_html)
        self.assertNotIn('<input checked="checked" id="id_is_active" name="is_active" type="checkbox" />', form_html)

    def test_user_form_dupe_username(self):
        LOGGER.debug("Test user form update")
        data = {
            'username': 'user1',
            'password': 'user2password',
            'email': 'user@user2.com',
            'first_name': 'user',
            'last_name': '2',
            'last_login': '2012-09-04 06:00',
            'date_joined': '2012-09-04 06:00',
            'manually_edited': False
        }
        form = UserForm(data)
        self.assertIn('already exists', form.errors['username'][0])

    def test_user_form_update(self):
        LOGGER.debug("Test user form create")
        data = {
            'username': 'user1',
            'first_name': 'user',
            'last_name': 'Two',
            'last_login': '2012-09-04 06:00',
            'date_joined': '2012-09-04 06:00',
            'manually_edited': False
        }
        form = UserForm(data, instance=self.user1)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEquals(self.user1.last_name, 'Two')


"""Django forms for the socialprofile application"""
from django import forms
from django.contrib.auth.models import User
from .models import SocialProfile
from django.utils.html import strip_tags
import logging

# pylint: disable=E1120,W0212

LOGGER = logging.getLogger(name='socialprofile.forms')


class UserForm(forms.ModelForm):
    """Form for editing the data that is part of the User model"""

    class Meta(object):
        """Configuration for the ModelForm"""
        model = User
        fields = {'username', 'first_name', 'last_name', 'email'}


class SocialProfileForm(forms.ModelForm):
    """Master form for editing the user's profile"""

    user = forms.IntegerField(widget=forms.HiddenInput, required=True)
    returnTo = forms.CharField(widget=forms.HiddenInput, required=False, initial='/')  # URI to Return to after save
    manually_edited = forms.BooleanField(widget=forms.HiddenInput, required=False, initial=True)

    class Meta(object):
        """Configuration for the ModelForm"""
        model = SocialProfile
        fields = {'user', 'gender', 'url', 'image_url', 'description'}  # Don't let through for security reasons, user should be based on logged in user only

    def clean_description(self):
        """Automatically called by Django, this method 'cleans' the description, e.g. stripping HTML out of desc"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean_description")

        return strip_tags(self.cleaned_data['description'])

    def clean(self):
        """Automatically called by Django, this method 'cleans' the whole form"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean")

        if self.changed_data:
            self.cleaned_data['manually_edited'] = True



"""Django Models for SocialProfile App"""

# pylint: disable=C0111,E0202,W0613,W0141

from django.db.models.signals import post_save
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

import logging

LOGGER = logging.getLogger(name='socialprofile.models')


class SocialProfile(models.Model):
    """Main SocialProfile Object - Holds extra profile data retrieved from auth providers"""
    GENDER_CHOICES = (
        (_('male'), _('Male')),
        (_('female'), _('Female')),
        (_('other'), _('Other')),
        ('', '')
    )
    user = models.OneToOneField(User, related_name='social_profile', verbose_name=_("Social Profile"))
    gender = models.CharField(max_length=10, blank=True, choices=GENDER_CHOICES, verbose_name=_("Gender"))
    url = models.URLField(blank=True, verbose_name=_("Homepage"), help_text=_("Where can we find out more about you?"), max_length=500)
    image_url = models.URLField(blank=True, verbose_name=_("Avatar Picture"), max_length=500)
    description = models.TextField(blank=True, verbose_name=_("Description"), help_text=_("Tell us about yourself!"))
    manually_edited = models.BooleanField(default=False)

    class Meta(object):
        verbose_name = _("Social Profile")
        verbose_name_plural = _("Social Profiles")
        ordering = ['user__username']

    def __str__(self):
        return self.user.username

    @models.permalink
    def get_absolute_url(self):
        return 'sp_profile_other_view_page', [self.user.username]


def create_user_profile(sender, instance, created, **kwargs):
    """Creates a UserProfile Object Whenever a User Object is Created"""
    if created:
        SocialProfile.objects.create(user=instance)


post_save.connect(create_user_profile, sender=User)



"""Template tags for the socialprofile module"""

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.translation import ugettext as _

register = template.Library()


@register.filter
@stringfilter
def social_provider_name(provider_slug):
    """Decode name of Social Auth Provider to Friendly Name"""
    if provider_slug == 'google-oauth2':
        return _("Google")

    if provider_slug == 'twitter':
        return _("Twitter")

    if provider_slug == 'facebook':
        return _("Facebook")



"""Unit Tests for the socialprofile module"""

# pylint: disable=R0904, C0103

from django.test import TestCase

from django.contrib.auth.models import User
from socialprofile.models import SocialProfile
from social.apps.django_app.default.models import UserSocialAuth
import logging

LOGGER = logging.getLogger(name='socialprofile')


class SocialProfileTestCase(TestCase):
    """Test Case for Social Profile"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfile Tests setUp")
        self.user1 = User.objects.create_user('user1', 'user1@user1.com', 'user1password')
        self.sp1 = SocialProfile.objects.update(user=self.user1,
                                                gender="Male",
                                                url="http://test.com",
                                                description="Test User 1",
                                                image_url="http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )
        self.sa1 = UserSocialAuth.objects.create(user=self.user1, provider='google-oauth2', uid='user1@user1.com')
        self.sa2 = UserSocialAuth.objects.create(user=self.user1, provider='facebook', uid='user1@user1.com')
        self.sa2 = UserSocialAuth.objects.create(user=self.user1, provider='twitter', uid='user1@user1.com')

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("SocialProfile Tests tearDown")
        self.user1.delete()

    def test_redirect_urls(self):
        """Test that redirects kicking in when trying to go to secure page."""
        LOGGER.debug("SocialProfile Test Redirect URLs")
        response = self.client.get('/secure/', follow=True)
        self.assertRedirects(response, "http://testserver/socialprofile/select/?next=/secure/")

    def test_view_profile(self):
        """Test to see if profile for user1 can be viewed anon and logged in"""
        LOGGER.debug("Test GET /socialprofile/view/user1/ for anon user")
        anon_view_response = self.client.get('/socialprofile/view/user1/')
        self.assertContains(anon_view_response, "Test User 1")

        LOGGER.debug("Test GET /socialprofile/ for anon user")
        anon_view_generic_response = self.client.get('/socialprofile/')
        self.assertEqual(404, anon_view_generic_response.status_code)

        LOGGER.debug("Test GET /socialprofile/view/ for anon user")
        anon_view_generic_response_2 = self.client.get('/socialprofile/view/')
        self.assertEqual(404, anon_view_generic_response_2.status_code)

        LOGGER.debug("Test GET /socialprofile/view/user1/ for logged in user")
        self.client.login(username='user1', password='user1password')
        logged_in_view_response = self.client.get('/socialprofile/view/user1/')
        self.assertContains(logged_in_view_response, "Test User 1")

        LOGGER.debug("Test GET /socialprofile/ for logged in user")
        logged_in_view_generic_response = self.client.get('/socialprofile/')
        self.assertContains(logged_in_view_generic_response, "Test User 1")

        LOGGER.debug("Test GET /socialprofile/view/ for logged in user")
        logged_in_view_generic_response_2 = self.client.get('/socialprofile/view/')
        self.assertEqual(404, logged_in_view_generic_response_2.status_code)

        LOGGER.debug("Test GET /socialprofile/view/ for logged in user")
        logged_in_view_generic_response_2 = self.client.get('/socialprofile/view/')
        self.assertEqual(404, logged_in_view_generic_response_2.status_code)

        LOGGER.debug("Test POST to /socialprofile/view/ for logged in user")
        logged_in_view_post_response = self.client.post('/socialprofile/', {'user': 1}, follow=True)
        self.assertEqual(405, logged_in_view_post_response.status_code)  # HTTP POST Not Allowed

    def test_socialprofile_permalink(self):
        """Test the permalink method of SocialProfile"""
        LOGGER.debug("Test SocialProfile Permalink")
        profile = SocialProfile.objects.get(user=self.user1)
        permalink = profile.get_absolute_url()
        self.assertEqual("/socialprofile/view/user1/", permalink)
        anon_view_response = self.client.get(permalink)
        self.assertContains(anon_view_response, "Test User 1")

    def test_edit_profile(self):
        """Test to see if profile for user1 can be edited anon and logged in"""
        LOGGER.debug("Test GET /socialprofile/edit/user1/ for anon user")
        anon_edit_response = self.client.get('/socialprofile/edit/user1/')
        self.assertEqual(404, anon_edit_response.status_code)

        LOGGER.debug("Test GET /socialprofile/edit/ for anon user")
        anon_edit_response_2 = self.client.get('/socialprofile/edit/')
        self.assertRedirects(anon_edit_response_2, "http://testserver/socialprofile/select/?next=/socialprofile/edit/")

        LOGGER.debug("Test POST /socialprofile/edit/ for anon user")
        anon_edit_response_2 = self.client.post('/socialprofile/edit/', {'user': 1}, follow=True)
        self.assertRedirects(anon_edit_response_2, "http://testserver/socialprofile/select/?next=/socialprofile/edit/")

        LOGGER.debug("Test GET /socialprofile/edit/user1/ for logged in user")
        self.client.login(username='user1', password='user1password')
        anon_edit_response = self.client.get('/socialprofile/edit/user1/')
        self.assertEqual(404, anon_edit_response.status_code)

        LOGGER.debug("Test GET /socialprofile/edit/ for logged in user")
        logged_in_edit_response = self.client.get('/socialprofile/edit/')
        self.assertContains(logged_in_edit_response, "Test User 1")

        LOGGER.debug("Test POST /socialprofile/edit/ for logged in user")
        post_data = {
            'user': 1,
            'username': 'user2',
            'email': 'user1@test.com',
            'first_name': 'Test',
            'last_name': 'User',
            'description': "Test1 User",
            'image_url': 'http://foo.com',
            'url': 'http://user1.com',
            'returnTo': '/secure/'
        }
        logged_in_edit_response_2 = self.client.post('/socialprofile/edit/', post_data, follow=True)
        self.assertContains(logged_in_edit_response_2, "updated")
        user = User.objects.get(username='user2')
        self.assertEqual('user2', str(user.social_profile))

        LOGGER.debug("Test Invalid Form Error")
        post_data = {
            'username': 'user2',
            'email': 'user1@test.com',
            'gender': 'Robot',
            'first_name': 'Test',
            'last_name': 'User',
            'description': "Test1 User",
            'image_url': 'http://foo.com',
            'url': 'http://user1.com',
            'returnTo': '/secure/'
        }
        logged_in_edit_response_3 = self.client.post('/socialprofile/edit/', post_data, follow=True)
        self.assertContains(logged_in_edit_response_3, "NOT")


    def test_delete_user(self):
        """Test the views that enable deleting users/socialprofiles"""
        LOGGER.debug("Test GET /socialprofile/delete/ for logged in user")
        self.client.login(username='user1', password='user1password')
        logged_in_delete_response = self.client.get('/socialprofile/delete/')
        self.assertContains(logged_in_delete_response, "Really Delete")

        LOGGER.debug("Test POST to /socialprofile/delete/ for logged in user")
        logged_in_delete_post_response = self.client.post('/socialprofile/delete/', {'user': 1}, follow=True)
        self.assertRedirects(logged_in_delete_post_response, '/')

        LOGGER.debug("Test GET /socialprofile/view/user1/ for deleted user")
        deleted_response = self.client.get('/socialprofile/view/user1/')
        # LOGGER.debug(deleted_response)
        self.assertEqual(404, deleted_response.status_code)








"""Django Views for the socialprofile module"""

# pylint: disable=R0901,R0904

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.http import Http404, HttpResponseRedirect
from django.views.generic import DeleteView, TemplateView
from django.utils.translation import ugettext_lazy as _
from .forms import SocialProfileForm, UserForm

import logging

LOGGER = logging.getLogger(name='socialprofile')

DEFAULT_RETURNTO_PATH = getattr(settings, 'DEFAULT_RETURNTO_PATH', '/')


class SelectAuthView(TemplateView):
    """
    Lets users choose how they want to request access.

    url: /select
    """
    template_name = 'socialprofile/sp_account_select.html'

    def get_context_data(self, **kwargs):
        """Ensure that 'next' gets passed along"""
        LOGGER.debug('socialprofile.views.SelectAuthView.get_context_data')

        next_url = self.request.GET.get(REDIRECT_FIELD_NAME, DEFAULT_RETURNTO_PATH)

        context = super(SelectAuthView, self).get_context_data(**kwargs)
        context['next_param'] = REDIRECT_FIELD_NAME
        context['next_url'] = next_url
        return context


class SocialProfileView(TemplateView):
    """
    Profile View Page

    url: /profile/view
    """
    template_name = 'socialprofile/sp_profile_view.html'

    http_method_names = {'get'}

    def get_context_data(self, **kwargs):
        """Load up the default data to show in the display form."""
        LOGGER.debug("socialprofile.views.SocialProfileView.get_context_data")
        username = self.kwargs.get('username')
        if username:
            user = get_object_or_404(User, username=username)
        elif self.request.user.is_authenticated():
            user = self.request.user
        else:
            raise Http404  # Case where user gets to this view anonymously for non-existent user

        return_to = self.request.GET.get('returnTo', DEFAULT_RETURNTO_PATH)

        sp_form = SocialProfileForm(instance=user.social_profile)
        user_form = UserForm(instance=user)

        sp_form.initial['returnTo'] = return_to

        return {'sp_form': sp_form, 'user_form': user_form}


class SocialProfileEditView(SocialProfileView):
    """
    Profile Editing View

    url: /profile/edit
    """

    template_name = 'socialprofile/sp_profile_edit.html'

    http_method_names = {'get', 'post'}

    def post(self, request, *args, **kwargs):
        user_form = UserForm(request.POST, instance=request.user)
        sp_form = SocialProfileForm(request.POST, instance=request.user.social_profile)

        if user_form.is_valid() & sp_form.is_valid():
            user_form.save()
            sp_form.save()
            messages.add_message(self.request, messages.INFO, _('Your profile has been updated.'))
            return HttpResponseRedirect(sp_form.cleaned_data.get('returnTo', DEFAULT_RETURNTO_PATH))
        else:
            messages.add_message(self.request, messages.INFO, _('Your profile has NOT been updated.'))
            return self.render_to_response({'sp_form': sp_form, 'user_form': user_form})


class DeleteSocialProfileView(DeleteView):
    """
    Account Delete Confirm Modal View

    url: /delete
    """

    success_url = reverse_lazy('sp_logout_page')

    template_name = "socialprofile/sp_delete_account_modal.html"

    model = User

    def get_object(self, queryset=None):
        """Get the object that we are going to delete"""
        return self.request.user



"""Django Admin Site configuration for socialprofiles"""

# pylint: disable=R0901,R0904

from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
from .models import SocialProfile


class SocialProfileInline(admin.TabularInline):
    """Sets up UserProfile to be inline editable along with users"""
    model = SocialProfile
    fk_name = 'user'
    max_num = 1


class CustomUserAdmin(UserAdmin):
    """Sets up the custom user admin display"""
    inlines = [SocialProfileInline, ]
    list_display = ('username', 'email', 'first_name', 'last_name', 'is_staff', 'is_active')


admin.site.unregister(User)
admin.site.register(User, CustomUserAdmin)


"""Python Social Auth Pipeline Extensions"""

from social.backends.google import GoogleOAuth2
from social.backends.twitter import TwitterOAuth
from social.backends.facebook import Facebook2OAuth2


def socialprofile_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Routes the extra values call to the appropriate back end"""
    if type(backend) is GoogleOAuth2:
        google_extra_values(backend, details, response, uid, user, *args, **kwargs)

    if type(backend) is TwitterOAuth:
        twitter_extra_values(backend, details, response, uid, user, *args, **kwargs)

    if type(backend) is Facebook2OAuth2:
        facebook_extra_values(backend, details, response, uid, user, *args, **kwargs)


def google_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Google Auth"""
    profile = user.social_profile
    if not profile.manually_edited:
        user.last_name = response.get('name').get('familyName', '')
        user.first_name = response.get('name').get('givenName', '')
        user.save()

        profile.gender = response.get('gender', '')
        profile.image_url = response.get('image').get('url', '')
        profile.url = response.get('url', '')
        profile.description = response.get('occupation', '')

        profile.save()


def facebook_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Facebook Auth"""
    profile = user.social_profile
    if not profile.manually_edited:
        user.last_name = response.get('last_name', '')
        user.first_name = response.get('first_name', '')
        user.save()

        profile.gender = response.get('gender', '')
        profile.url = response.get('link', '')
        if response.get('picture', False):
            profile.image_url = response.get('picture').get('data').get('url', '')

        profile.save()

    return response


def twitter_extra_values(backend, details, response, uid, user, *args, **kwargs):
    profile = user.social_profile
    if not profile.manually_edited:
        try:
            first_name, last_name = response.get('name', '').split(' ', 1)
        except:
            first_name = response.get('name', '')
            last_name = ''
        user.last_name = last_name
        user.first_name = first_name
        user.save()

        profile.url = response.get('url', '')
        profile.image_url = response.get('profile_image_url_https', '')
        profile.description = response.get('description', '')

        profile.save()

    return response


"""
    Master URL Pattern List for the application.  Most of the patterns here should be top-level
    pass-offs to sub-modules, who will have their own urls.py defining actions within.
"""

# pylint: disable=W0401, W0614, E1120

from django.conf.urls import url, include
from .views import DeleteSocialProfileView, SelectAuthView, SocialProfileView, SocialProfileEditView
from django.views.decorators.cache import never_cache
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import logout

urlpatterns = (

    # Profile Self View
    url(r'^$', never_cache(SocialProfileView.as_view()), name="sp_profile_view_page"),

    # Profile Other View
    url(r'^view/(?P<username>\w+)/$', SocialProfileView.as_view(), name="sp_profile_other_view_page"),

    # Profile Edit
    url(r'^edit/$', never_cache(login_required(SocialProfileEditView.as_view())), name="sp_profile_edit_page"),

    # Select Sign Up Method
    url(r'^select/$', never_cache(SelectAuthView.as_view()), name="sp_select_page"),

    # Delete
    url(r'^delete/$', login_required(DeleteSocialProfileView.as_view()), name="sp_delete_page"),

    # Social Auth
    url('', include('social.apps.django_app.urls', namespace='social')),

    # Logout Page
    url(r'^logout/$', logout, kwargs={'next_page': "/"}, name="sp_logout_page"),

)



"""Unit Tests for the socialprofile module forms"""

# pylint: disable=R0904, C0103

from django.test import TestCase

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from socialprofile.models import SocialProfile
from social.apps.django_app.default.models import UserSocialAuth
from socialprofile.forms import SocialProfileForm, UserForm
from django.forms.models import model_to_dict
from socialprofile.pipeline import facebook_extra_values, google_extra_values, socialprofile_extra_values, twitter_extra_values
from social.backends.google import GoogleOAuth2
from social.backends.twitter import TwitterOAuth
from social.backends.facebook import Facebook2OAuth2

import logging

LOGGER = logging.getLogger(name='socialprofile.test_forms')


class SocialProfilePipelineTestCase(TestCase):
    """Test Case for Social Profile Pipeline"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfile Pipeline Tests setUp")
        self.user1 = User.objects.create_user('user1', 'user1@user1.com', 'user1password')
        self.user1.social_profile.gender = 'other'
        self.user1.social_profile.url = 'http://test.com'
        self.user1.social_profile.description = 'Test User 1'
        self.user1.social_profile.image_url = 'http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm'
        self.user1.save()
        self.sa1 = UserSocialAuth.objects.create(user=self.user1, provider='google-oauth2', uid='user1@user1.com')

    def test_socialprofile_pipeline_google(self):
        """Test editing executing pipeline methods in isolation for google"""
        LOGGER.debug("Test socialprofile pipeline Google")
        backend = GoogleOAuth2()
        response = {
            'name': {
                'familyName': 'User 1',
                'givenName': 'Test'
            },
            'gender': 'other',
            'image': {'url': 'http://image.url'},
            'occupation': 'User Description',
            'url': 'http://test.com'
        }

        socialprofile_extra_values(backend, {}, response, '1', self.user1)
        self.assertEquals(self.user1.social_profile.description, 'User Description')
        self.assertEquals(self.user1.social_profile.gender, 'other')
        self.assertEquals(self.user1.social_profile.image_url, 'http://image.url')
        self.assertEquals(self.user1.social_profile.url, 'http://test.com')

    def test_socialprofile_pipeline_facebook(self):
        """Test editing executing pipeline methods in isolation for facebook"""
        LOGGER.debug("Test socialprofile pipeline Facebook")
        backend = Facebook2OAuth2()
        response = {
            'name': {
                'last_name': 'User 1',
                'first_name': 'Test'
            },
            'gender': 'other',
            'picture': {'data': {'url': 'http://image.url'}},
            'link': 'http://test.com'
        }

        socialprofile_extra_values(backend, {}, response, '1', self.user1)
        self.assertEquals(self.user1.social_profile.gender, 'other')
        self.assertEquals(self.user1.social_profile.image_url, 'http://image.url')
        self.assertEquals(self.user1.social_profile.url, 'http://test.com')

    def test_socialprofile_pipeline_twitter(self):
        """Test editing executing pipeline methods in isolation for twitter"""
        LOGGER.debug("Test socialprofile pipeline Twitter")
        backend = TwitterOAuth()
        response = {
            'name': {
                'last_name': 'User 1',
                'first_name': 'Test'
            },
            'gender': 'other',
            'profile_image_url_https': 'http://image.url',
            'description': 'User Description',
            'url': 'http://test.com'
        }

        socialprofile_extra_values(backend, {}, response, '1', self.user1)
        self.assertEquals(self.user1.social_profile.description, 'User Description')
        self.assertEquals(self.user1.social_profile.gender, 'other')
        self.assertEquals(self.user1.social_profile.image_url, 'http://image.url')
        self.assertEquals(self.user1.social_profile.url, 'http://test.com')


from setuptools import setup

readme = open('README.rst').read()

setup(
    name="django-socialprofile",
    version="1.2.1",
    url='https://github.com/cyface/django-socialprofile',
    license='BSD',
    description="django-socialprofile enables users to manage their user profile built by logging in via a social \
    service such as Google, Twitter, or Facebook. It relies on python-socialauth.",
    author='Tim White',
    author_email='tim@cyface.com',
    packages=[
        'socialprofile',
        'socialprofile.templatetags',
    ],
    long_description=readme,
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'django>=1.8.3',
        'python-social-auth>=0.2.19',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
